@@ -80,6 +80,7 @@ constexpr char argumentsKey[] = "arguments";
 constexpr char settingsGroupKey[] = "LanguageClient";
 constexpr char clientsKey[] = "clients";
 constexpr char typedClientsKey[] = "typedClients";
+constexpr char outlineSortedKey[] = "outlineSorted";
 constexpr char mimeType[] = "application/language.client.setting";
 
 namespace LanguageClient {
@@ -193,7 +194,7 @@ LanguageClientSettingsPageWidget::LanguageClientSettingsPageWidget(LanguageClien
     auto addMenu = new QMenu;
     addMenu->clear();
     for (const ClientType &type : clientTypes()) {
-        auto action = new QAction(LanguageClientSettingsPage::tr("New %1").arg(type.name));
+        auto action = new QAction(type.name);
         connect(action, &QAction::triggered, this, [this, id = type.id]() { addItem(id); });
         addMenu->addAction(action);
     }
@@ -360,6 +361,7 @@ QList<BaseSettings *> LanguageClientSettingsPage::changedSettings() const
 void LanguageClientSettingsPage::addSettings(BaseSettings *settings)
 {
     m_model.insertSettings(settings);
+    m_changedSettings << settings->m_id;
 }
 
 void LanguageClientSettingsPage::enableSettings(const QString &id)
@@ -557,16 +559,22 @@ bool BaseSettings::isValid() const
 }
 
 Client *BaseSettings::createClient()
+{
+    return createClient(nullptr);
+}
+
+Client *BaseSettings::createClient(ProjectExplorer::Project *project)
 {
     if (!isValid() || !m_enabled)
         return nullptr;
-    BaseClientInterface *interface = createInterface();
+    BaseClientInterface *interface = createInterfaceWithProject(project);
     QTC_ASSERT(interface, return nullptr);
     auto *client = createClient(interface);
     client->setName(Utils::globalMacroExpander()->expand(m_name));
     client->setSupportedLanguage(m_languageFilter);
     client->setInitializationOptions(initializationOptions());
     client->setActivateDocumentAutomatically(true);
+    client->setCurrentProject(project);
     return client;
 }
 
@@ -679,6 +687,23 @@ void LanguageClientSettings::toSettings(QSettings *settings,
     settings->endGroup();
 }
 
+bool LanguageClientSettings::outlineComboBoxIsSorted()
+{
+    auto settings = Core::ICore::settings();
+    settings->beginGroup(settingsGroupKey);
+    bool sorted = settings->value(outlineSortedKey).toBool();
+    settings->endGroup();
+    return sorted;
+}
+
+void LanguageClientSettings::setOutlineComboBoxSorted(bool sorted)
+{
+    auto settings = Core::ICore::settings();
+    settings->beginGroup(settingsGroupKey);
+    settings->setValue(outlineSortedKey, sorted);
+    settings->endGroup();
+}
+
 bool StdIOSettings::applyFromSettingsWidget(QWidget *widget)
 {
     bool changed = false;
@@ -709,7 +734,7 @@ bool StdIOSettings::isValid() const
 QVariantMap StdIOSettings::toMap() const
 {
     QVariantMap map = BaseSettings::toMap();
-    map.insert(executableKey, m_executable);
+    map.insert(executableKey, m_executable.toVariant());
     map.insert(argumentsKey, m_arguments);
     return map;
 }
@@ -717,7 +742,7 @@ QVariantMap StdIOSettings::toMap() const
 void StdIOSettings::fromMap(const QVariantMap &map)
 {
     BaseSettings::fromMap(map);
-    m_executable = map[executableKey].toString();
+    m_executable = Utils::FilePath::fromVariant(map[executableKey]);
     m_arguments = map[argumentsKey].toString();
 }
 
@@ -728,15 +753,15 @@ QString StdIOSettings::arguments() const
 
 Utils::CommandLine StdIOSettings::command() const
 {
-    return Utils::CommandLine(Utils::FilePath::fromUserInput(m_executable),
-                              arguments(),
-                              Utils::CommandLine::Raw);
+    return Utils::CommandLine(m_executable, arguments(), Utils::CommandLine::Raw);
 }
 
-BaseClientInterface *StdIOSettings::createInterface() const
+BaseClientInterface *StdIOSettings::createInterfaceWithProject(ProjectExplorer::Project *project) const
 {
     auto interface = new StdIOClientInterface;
     interface->setCommandLine(command());
+    if (project)
+        interface->setWorkingDirectory(project->projectDirectory());
     return interface;
 }
 
@@ -967,16 +992,16 @@ StdIOSettingsWidget::StdIOSettingsWidget(const StdIOSettings *settings, QWidget
     mainLayout->addWidget(m_executable, baseRows, 1);
     mainLayout->addWidget(new QLabel(tr("Arguments:")), baseRows + 1, 0);
     m_executable->setExpectedKind(Utils::PathChooser::ExistingCommand);
-    m_executable->setPath(QDir::toNativeSeparators(settings->m_executable));
+    m_executable->setFilePath(settings->m_executable);
     mainLayout->addWidget(m_arguments, baseRows + 1, 1);
 
     auto chooser = new Utils::VariableChooser(this);
     chooser->addSupportedWidget(m_arguments);
 }
 
-QString StdIOSettingsWidget::executable() const
+Utils::FilePath StdIOSettingsWidget::executable() const
 {
-    return m_executable->filePath().toString();
+    return m_executable->filePath();
 }
 
 QString StdIOSettingsWidget::arguments() const
